/* *************************************************************************
                          box.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002   
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "box.h"
#include "rendererror.h"
#include "material3d.h"

HitStatistics Box::stat("Box","Tests","Hits");

Intersection* Box::intersectUnBounded(const Ray &r) {
    return intersectUnBounded(r, INTERSECTION_INFINIT_TIME);	
}

Intersection* Box::intersectUnBounded(const Ray &r, DBL maxtime) {

    stat.eval();

    DBL tmin, tmax = maxtime;
    boxside smin = X, smax = X;

    Vector3 direction = r.getDirection();
    Vector3 origin = r.getOrigin();

    // calculate x time interval
    if (direction.a[0]>0) {

        DBL t = (-origin.a[0]+boxmax.a[0]) / direction.a[0];
        if (t<INTERSECTION_TIME_EPSILON) return 0;  if (t<tmax) tmax = t;
        tmin = (-origin.a[0]+boxmin.a[0]) / direction.a[0];
        if (tmin<INTERSECTION_TIME_EPSILON) tmin = INTERSECTION_TIME_EPSILON;
        else if (tmin>tmax) return 0;

    } else if (direction.a[0]!=0){

        DBL t = (-origin.a[0]+boxmin.a[0]) / direction.a[0];
        if (t<INTERSECTION_TIME_EPSILON) return 0;  if (t<tmax) tmax = t;
        tmin = (-origin.a[0]+boxmax.a[0]) / direction.a[0];
        if (tmin<INTERSECTION_TIME_EPSILON) tmin = INTERSECTION_TIME_EPSILON;
        else if (tmin>tmax) return 0;

    } else {

        if (origin.a[0]<boxmin.a[0] || origin.a[0]>boxmax.a[0] ) return 0;
        tmin = INTERSECTION_TIME_EPSILON;

    }

    // calculate y time interval
    if (direction.a[1]>0) {

        DBL t = (-origin.a[1]+boxmax.a[1]) / direction.a[1]; //y max
        if (t<tmin) return 0;  if (t<tmax) {tmax = t; smax = Y;}
        t = (-origin.a[1]+boxmin.a[1]) / direction.a[1]; //y min
        if (t>tmax) return 0;  if (t>tmin) {tmin = t; smin = Y;}

    } else if (direction.a[1]!=0) {

        DBL t = (-origin.a[1]+boxmin.a[1]) / direction.a[1]; //y max
        if (t<tmin) return 0;  if (t<tmax) {tmax = t; smax = Y;}
        t = (-origin.a[1]+boxmax.a[1]) / direction.a[1]; //y min
        if (t>tmax) return 0;  if (t>tmin) {tmin = t; smin = Y;}

    } else {
        if (origin.a[1]<boxmin.a[1] || origin.a[1]>boxmax.a[1] ) return 0;
    }

    // calculate z time interval
    if (direction.a[2]>0) {

        DBL t = (-origin.a[2]+boxmax.a[2]) / direction.a[2];  // z max
        if (t<tmin) return 0;  if (t<tmax) {tmax = t; smax = Z;}
        t = (-origin.a[2]+boxmin.a[2]) / direction.a[2];  // z min
        if (t>tmax) return 0;  if (t>tmin) {tmin = t; smin = Z;}

    } else if(direction.a[2]!=0) {

        DBL t = (-origin.a[2]+boxmin.a[2]) / direction.a[2]; // z max
        if (t<tmin) return 0;  if (t<tmax) {tmax = t; smax = Z;}
        t = (-origin.a[2]+boxmax.a[2]) / direction.a[2]; // z min
        if (t>tmax) return 0;  if (t>tmin) {tmin = t; smin = Z;}

    } else {
        
        if (origin.a[2]>boxmin.a[2] || origin.a[2]<boxmax.a[2] ) return 0;
        
    }
    
    if (tmin == INTERSECTION_TIME_EPSILON) {
        if (tmax == maxtime) {
            return 0;
        } else {
            stat.success();
            return new BoxIntersection(this,r,tmax,INTERSECTION_INFINIT_TIME,smax,NONE);            
        }
    }

    if (tmax == maxtime) tmax = INTERSECTION_INFINIT_TIME;

    stat.success();
    return new BoxIntersection(this,r,tmin,tmax,smin,smax);
  
}

bool Box::isInsideAt(const Vector3 &a) const {
    return (a.a[0]>=boxmin.a[0] && a.a[1]>=boxmin.a[1] && a.a[2]>=boxmin.a[2] &&
            a.a[0]<=boxmax.a[0] && a.a[1]<=boxmax.a[1] && a.a[2]<=boxmax.a[2])
           ^inversed;
}

void Box::prepareToRender() {
    
    if (transformation == 0) {
        delete(boundingbox);
        boundingbox = 0;
    }
    
}

void Box::BoxIntersection::next() {
    currenttime = secondtime;
    currentside = secondside;
    secondtime = INTERSECTION_INFINIT_TIME;
}

Vector3 Box::BoxIntersection::normal() {
    switch(currentside) {
        case(X): return Vector3(1,0,0);
        case(Y): return Vector3(0,1,0);
        case(Z): return Vector3(0,0,1);
        default: throw(RenderError("Invalid Box side"));
    }
}

