/* *************************************************************************
                          cameraray.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002   
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */      

#include "cameraray.h"
#include "scene.h"
#include "texture.h"
#include "rendererror.h"

HitStatistics CameraRay::statcount("Rays","Count","Saved");
HitStatistics CameraRay::statref("Rays","Reflected","Refracted");

CameraRay::CameraRay(const Vector3 o, const Vector3 d, Scene* s, InsideList* l):
  Ray(o,d), myscene(s), inside(l), hit(0), htime(0), ltime(0), hitstatus(NONE),
  tracelevel(myscene->maxtracelevel), weight(1.0/myscene->colourprecision) {

    statcount.eval();
    direction.normalize();

}

CameraRay::~CameraRay() {
    delete(hit);
    delete(inside);  
}

void CameraRay::getIntersectionInfo() {
    ltime = htime;
    htime = hit->currentTime();
    hpoint = positionAtTime(hit->currentTime());
    norm = hit->normal();
    norm.normalize();
    if (Vector3::dotProd(norm,direction)>0) norm.scale(-1);
}

bool CameraRay::shoot() {

    if (tracelevel<=0  || weight < 1) {

        statcount.success();
        return false;
        
    }

    if (hitstatus == NONE || hitstatus == OLD) {
    
        delete(hit);
        hit = myscene->object.intersectUnBounded(*this);
        hitstatus = OWN;
        if (hit == 0) return false;
        getIntersectionInfo();
        return true;

    } else { // hitstatus == OWN

        if (hit == 0) return false;
        hit->next();
        if (hit->currentTime() == INTERSECTION_INFINIT_TIME) return false;
        getIntersectionInfo();
        return true;
    
    }

}

bool CameraRay::refract(FLT weightmult) {

    const Object3D* hitobj = hit->hitObject();
    FLT m = inside->swap(hitobj);

    if (m != 1.0) {

        statref.success();

        htime = 0;

        // calculate new direction
        DBL cosa = -Vector3::dotProd(direction,norm);
        DBL x = m*m*(cosa*cosa-1);

        if (x<-1.0) {
            // total reflection
            inside->add(hitobj);
            return true;
        }
     
        tracelevel--;
        
        if (hitstatus == NONE) throw(RenderError("can't refract new rays!"));
        hitstatus = OLD;
    
        origin = hpoint;
        direction.scale(m);
        DBL s = -(sqrt(x+1)-m*cosa);
        Vector3 nscal = norm.scaled(s);
        direction.add(nscal);
        direction.normalize();
    }

    weight *= weightmult;
    return false;
    
}

void CameraRay::reflect(FLT weightmult) {

    statref.eval();

    htime = 0;

    weight *= weightmult;

    DBL x = -2*Vector3::dotProd(norm,direction);
    Vector3 reflecteddirection = norm.scaled(x);
    reflecteddirection.add(direction);
    origin = hpoint;
    direction = reflecteddirection;
    hitstatus = OLD;
    tracelevel--;
  
}


Colour3 CameraRay::evaluateMedia(Colour3& a) const {

    a = Colour3(0);
    Media3D* thismedia = inside->getMedia();
    if (thismedia) {
        return thismedia->sample(ltime,htime,*this,a);
    } else return Colour3(0);

}
