/* *************************************************************************
                            matrix.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002    
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "matrix.h"
#include<cmath>

void Matrix::transpose() {

    DBL tmp=a[0][1];
    a[0][1]=a[1][0];
    a[1][0]=tmp;
        
        tmp=a[0][2];
        a[0][2]=a[2][0];
        a[2][0]=tmp;
        
        tmp=a[1][2];
        a[1][2]=a[2][1];
        a[2][1]=tmp;

}

void Matrix::applyFromLeft(const Matrix &m) {
    Matrix tmp = Matrix(*this);
    zero();
    for(int i=0; i<3; ++i)
        for(int j=0; j<3; ++j)
            for(int k=0; k<3; ++k)
                a[i][j] += tmp.a[i][k] * m.a[k][j];
}

Matrix Matrix::generateRotationMatrix(Vector3 v) {
    v.scale(PI/180);
    double sinx = sin(v.a[0]);
    double siny = sin(v.a[1]);
    double sinz = sin(v.a[2]);
        
    double cosx = cos(v.a[0]);
    double cosy = cos(v.a[1]);
    double cosz = cos(v.a[2]);
        
    Matrix rot;
    rot.a[0][0] = cosy*cosz;
    rot.a[0][1] = cosy*sinz;
    rot.a[0][2] =-siny;
        
    rot.a[1][0] = cosz*sinx*siny-cosx*sinz;
    rot.a[1][1] = cosx*cosz+sinx*siny*sinz;
    rot.a[1][2] = cosy*sinx;
        
    rot.a[2][0] = cosx*cosz*siny+sinx*sinz;
    rot.a[2][1] =-cosz*sinx+cosx*siny*sinz;
    rot.a[2][2] = cosx*cosy;

    rot.transpose();
    return rot;
}

Matrix Matrix::generateScaleMatrix(const Vector3 &v) {
    Matrix m;
    m.a[0][0]=v.a[0]; m.a[0][1]=0; m.a[0][2]=0;
    m.a[1][0]=0; m.a[1][1]=v.a[1]; m.a[1][2]=0;
    m.a[2][0]=0; m.a[2][1]=0; m.a[2][2]=v.a[2];   
    return m;
}
