/* *************************************************************************
                          media3d.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002 
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "media3d.h"
#include "random.h"
#include "scene.h"
#include "lightlist.h"

/*#include <iostream>
using std::cout;*/

HitStatistics Media3D::evaluations("Media","Evaluations","Samples");

Colour3 Media3D::simps(const Colour3& c1, const Colour3& c2, const Colour3& c3) {

  // Simpson: c1 + 4*c2 + c3
  Colour3 ret = c2.scaled(4);
  ret.add(c1); ret.add(c3);
  return ret;

}

Colour3 Media3D::sampleRecursive(const CameraRay& ray, DBL l, DBL m, DBL r,
/*  const Colour3& em_l_o, const Colour3& em_m_o, const Colour3& em_r_o,*/
  const Colour3& ab_l, const Colour3& ab_m, const Colour3& ab_r,
  Colour3& absorb) {

    DBL h = (r-l)/4;
    CLR block_err = 0;
    CLR eval_err;

    Colour3 em_l = evaluateEmit(ray.positionAtTime(l),ray,eval_err,h); block_err += eval_err;
    Colour3 em_m = evaluateEmit(ray.positionAtTime(m),ray,eval_err,h); block_err += eval_err*2;
    Colour3 em_r = evaluateEmit(ray.positionAtTime(r),ray,eval_err,h); block_err += eval_err;

    // evaluate new point between l and m
    DBL ml = (l+m)/2;
    Vector3 p = ray.positionAtTime(ml);
    Colour3 em_ml = evaluateEmit(p,ray,eval_err,h); block_err += eval_err*4;
    Colour3 ab_ml = evaluateAbsorb(p);

    // evaluate new point between m and r
    DBL mr = (r+m)/2;
    p = ray.positionAtTime(mr);
    Colour3 em_mr = evaluateEmit(p,ray,eval_err,h); block_err += eval_err*4;
    Colour3 ab_mr = evaluateAbsorb(p);
    block_err /= 12;

    // calculate absorption integrals for both halfs
    Colour3 ab_im = simps(ab_l,ab_ml,ab_m).scaled(-h/3);
    Colour3 ab_ir = simps(ab_m,ab_mr,ab_r).scaled(-h/3);
    Colour3 ab_tot = Colour3::add(ab_im,ab_ir);

    ab_im.exp();
    ab_ir.exp();

    // approximate the integral with the points l,m,r
    Colour3 f_m = em_m.scaled(ab_im);
    Colour3 f_r = em_r.scaled(ab_im).scaled(ab_ir);
    Colour3 i1 = simps(em_l, f_m, f_r).scaled(h/1.5);

    // do a more precise approximation using all 5 points
    Colour3 f_ml = em_ml.scaled( Colour3::sqrt(ab_im) );
    Colour3 f_mr = em_mr.scaled(ab_im).scaled( Colour3::sqrt(ab_ir) );
    Colour3 i2 = Colour3::add(f_ml,f_mr); i2.scale(2);
    i2.add(f_m); i2.scale(2); i2.add(em_l); i2.add(f_r); i2.scale(h/3);

    // do Romberg step
    Colour3 i3 = i2.scaled(16); i3.sub(i1); i3.scale(1.0/15);

    // estimate error
    Colour3 absorb_exp = Colour3::exp(absorb);
    DBL err = Colour3::sub(i3,i2).scaled( absorb_exp ).norm();
    block_err *= absorb_exp.norm();

    if (err<coltolerance &&  block_err*h<colblocktolerance) {
        absorb.add(ab_tot);
        return i3.scaled(absorb_exp);
    }

    // do recursion
    Colour3 left = sampleRecursive(ray, l,ml,m, /*em_l,em_ml,em_m,*/ ab_l,ab_m,ab_r, absorb);
    Colour3 right = sampleRecursive(ray, m,mr,r, /*em_m,em_mr,em_r,*/ ab_m,ab_mr,ab_r, absorb);
    return Colour3::add(left,right);
    
}

Colour3 Media3D::sample(DBL start, DBL end, const CameraRay& r, Colour3& totalabsorb) {

    evaluations.eval();

    Vector3 p_l = r.positionAtTime(start);
    DBL m = start + (end-start)/2*Random::jitterFactor(jitter);
    Vector3 p_m = r.positionAtTime(m);
    Vector3 p_r = r.positionAtTime(end);

/*    CLR block_err;
    Colour3 em_l = evaluateEmit(p_l,r,block_err,(end-start)/4);
    Colour3 em_r = evaluateEmit(p_r,r,block_err,(end-start)/4); 
    Colour3 em_m = evaluateEmit(p_m,r,block_err,(end-start)/4); */

    Colour3 ab_l = evaluateAbsorb(p_l);
    Colour3 ab_m = evaluateAbsorb(p_m);
    Colour3 ab_r = evaluateAbsorb(p_r);

    return sampleRecursive(r,start,m,end, /*em_l,em_m,em_r,*/ ab_l,ab_m,ab_r,totalabsorb);

}

void Media3D::sampleRecursive(const LightRay& ray, DBL l, DBL m, DBL r,
  const Colour3& ab_l, const Colour3& ab_m, const Colour3& ab_r, Colour3& absorb) {

//    cout << "["<<l<<","<<r<<"]";

    DBL h = (r-l)/4;
    DBL ml = l+h;
    DBL mr = r-h;

    Colour3 ab_ml = evaluateAbsorb( ray.positionAtTime(ml) );
    Colour3 ab_mr = evaluateAbsorb( ray.positionAtTime(mr) );

    // approximate the integral with the points l,m,r
    Colour3 i1  = simps(ab_l,ab_m,ab_r).scaled(h/1.5);

    // do a more precise approximation using all 5 points
    Colour3 i2 = Colour3::add( simps(ab_l,ab_ml,ab_m), simps(ab_m,ab_mr,ab_r) ).scaled(h/3);

    // do Romberg step
    Colour3 i3 = i2.scaled(16); i3.sub(i1); i3.scale(1.0/15);

    // estimate error
    DBL err = Colour3::sub(i3,i2).norm();

//    cout << " i1: "<<i1.norm()<< " i2: "<<i2.norm() << " i3: "<<i3.norm() << "err: " << err << "\n";
    
    if (err<coltolerance) {
        absorb.add(i3);
        return;
    }

//    cout << "rec\n";
    // do recursion
    sampleRecursive(ray, l,ml,m, ab_l,ab_m,ab_r, absorb);
    sampleRecursive(ray, m,mr,r, ab_m,ab_mr,ab_r, absorb);

}


void Media3D::sample(DBL start, DBL end, const LightRay& r, Colour3& totalabsorb) {

    DBL m = start + (end-start)/2*Random::jitterFactor(jitter);
    Colour3 ab_l = evaluateAbsorb( r.positionAtTime(start) );
    Colour3 ab_m = evaluateAbsorb( r.positionAtTime(m) );
    Colour3 ab_r = evaluateAbsorb( r.positionAtTime(end) );

    sampleRecursive(r, start,m,end, ab_l,ab_m,ab_r, totalabsorb);

}



Media3DSingle::Media3DSingle(
  const Colour3& emitting, const Colour3& absorbing,
  const Colour3& scattering, scattertype t, const Scene* s,
  DBL jitter, CLR tol1, CLR tol2):
  Media3D(jitter,tol1,tol2), emit(emitting), absorb(absorbing), scatter(scattering),
  stype(t), density(0), scene(s) {

    scatternorm = scatter.norm();

}

Media3DSingle::~Media3DSingle() {
    delete(density);
}

Media3DSingle::Media3DSingle(const Media3DSingle& m): Media3D(m), emit(m.emit), absorb(m.absorb), scatter(m.scatter),
  stype(m.stype), density(0), scene(m.scene), scatternorm(m.scatternorm) {

    if (m.density) {
        density = m.density->copy();
    }


}

Colour3 Media3DSingle::evaluateEmit(const Vector3& p, const CameraRay& r, CLR& maxerror, DBL precision) const {

    evaluations.evalSuccess();

    // calculate emitting contribution:
    Colour3 ret = emit;

    // calculate scattering contribution:
    if (stype != NONE) {

        // get list with all relevant LightSources
        LightList* scatterlist;
        scatterlist = scene->lightcache.lookup(p, maxerror, precision);

        maxerror *= scatternorm;

        FLT maxfact = 0;

        // loop over all LightSources and get the amouth of light comming here
        LightList* it = scatterlist;
        while(it != 0) {

            LightRay l = it->light->shineTo(p, 0);

            FLT factor;
            if (stype != ISOTROPIC) {
                DBL angle = Vector3::dotProd(r.getDirection(),l.getDirection())/r.getDirection().length()/l.getDirection().length();
                switch(stype) {
                    // taken from POV-Ray 3.5, media.cpp
                    case RAYLEIGH:
                        factor = 0.799372013f * (1.0f + pow(angle,2));
                        break;
                    case MIE_HAZY:
                        factor = 0.576655375f * (1.0f + 9.0f * pow(0.5 * (1.0 + angle), 8.0));
                        break;
                    case MIE_MURKY:
                        factor = 0.495714547f * (1.0f + 50.0f * pow(0.5 * (1.0 + angle), 32.0));
                        break;
                    /*
                        case HENYEY_GREENSTEIN:
                            factor *=
                    */

                    default:
                        factor =1;
                }
                if (isnan(factor)) factor = 0;
            } else factor =1;

//            std::cout << factor<<" ";
            if (factor>maxfact) maxfact = factor;
            ret.add(it->colour.scaled(factor).scaled(scatter));
            it = it->next;

        }

        maxerror *= maxfact;
        if (scene->lightcache.itype != LightCache::NEAREST && scene->lightcache.itype != LightCache::EXACT) delete(scatterlist);

    }

   if (density) {
        Vector3 loc = p;
        density->transformPointIn(loc);
        Colour3 densityfactor = density->evaluateAt(loc);
        ret.scale(densityfactor);
        maxerror *= densityfactor.norm();
    }

    return ret;

}

Colour3 Media3DSingle::evaluateAbsorb(const Vector3& p) const {

    if (density) {
        Vector3 loc = p;
        density->transformPointIn(loc);
        Colour3 densityfactor = density->evaluateAt(loc);
        return absorb.scaled(densityfactor);

    }

    return absorb;

}


Media3DMultiple::Media3DMultiple(DBL jitteramouth, CLR tol1, CLR tol2): Media3D(jitteramouth,tol1,tol2), list() {};

Media3DMultiple::Media3DMultiple(const Media3D* m): Media3D(*m), list() {
    list.push_back(m);
}

Media3DMultiple::~Media3DMultiple() {
    std::vector<const Media3D*>::const_iterator it = list.begin();
    for(; it!=list.end(); ++it) {
        delete(*it);
    }
}

Media3DMultiple::Media3DMultiple(const Media3DMultiple& m): Media3D(m), list() {

    std::vector<const Media3D*>::const_iterator it = m.list.begin();
    for(; it!=m.list.end(); ++it) {
        list.push_back( (*it)->copy() );
    }

}

Colour3 Media3DMultiple::evaluateEmit(const Vector3& p, const CameraRay& r, CLR& maxerror, DBL precision) const {

    Colour3 ret(0);
    std::vector<const Media3D*>::const_iterator it = list.begin();
    for(; it!=list.end(); ++it) {
        ret.add( (*it)->evaluateEmit(p,r,maxerror,precision) );
    }
    return ret;

}

Colour3 Media3DMultiple::evaluateAbsorb(const Vector3& p) const {

    Colour3 ret(0);
    std::vector<const Media3D*>::const_iterator it = list.begin();
    for(; it!=list.end(); ++it) {
        ret.add( (*it)->evaluateAbsorb(p) );
    }
    return ret;

}
  
