/* *************************************************************************
                            sphere.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002    
    copyright            : (C) 2002 by Micha Riser
    email                : mriser@gmx.net
 ************************************************************************* */

/* *************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "sphere.h"
#include "material3d.h"

HitStatistics Sphere::stat("Sphere","Tests","Hits");

void Sphere::rotate(const Vector3& v) {

/*    if (transformation) Object3D::rotate(v); else {
        Matrix rot = Matrix::generateRotationMatrix(v);
        translation.applyFromLeftTransposed(rot);
        calculateBoundingBox();
    }*/
    Object3D::rotate(v);

}

Intersection* Sphere::intersectUnBounded(const Ray &r)  {

    stat.eval();

    // *optimize me*
    DBL a=r.getDirection().lengthSq();
    DBL b=Vector3::dotProd(r.getOrigin(),r.getDirection());
    DBL c=r.getOrigin().lengthSq()-radiussq;
    DBL det=b*b-a*c;

    if (det<0) return 0;

    det = sqrt(det);
    DBL t1=-b-det;
    if (t1>INTERSECTION_TIME_EPSILON) {
        
        stat.success();    
        return new SphereIntersection(this, r,t1/a, (-b+det)/a );
        
    } else {
        DBL t2=-b+det;
        if (t2>INTERSECTION_TIME_EPSILON) {
            
            stat.success();
            return new SphereIntersection(this, r, t2/a, INTERSECTION_INFINIT_TIME);
            
        }
    }

    return 0;

}

Intersection* Sphere::intersectUnBounded(const Ray &r, DBL tmax) {

    stat.eval();

    // *optimize me*
    DBL a=r.getDirection().lengthSq();
    DBL b=Vector3::dotProd(r.getOrigin(),r.getDirection());
    DBL c=r.getOrigin().lengthSq()-radiussq;
    DBL det=b*b-a*c;

    if (det<0) return 0;

    det = sqrt(det);
    DBL t1 = (-b-det)/a;
    if (t1>INTERSECTION_TIME_EPSILON) {

        if (t1<tmax) {
            DBL t2 = (-b+det)/a;
            stat.success();            
            if (t2>tmax) 
                return new SphereIntersection(this, r,t1,INTERSECTION_INFINIT_TIME);
            else 
                return new SphereIntersection(this, r,t1,t2);
            
        }
        
    } else {

        DBL t2 = (-b+det)/a;
        if ((t2>1e-10)&&(t2<tmax)) {
            stat.success();    
            return new SphereIntersection(this, r,t2,INTERSECTION_INFINIT_TIME);
        }
        
    }
         
    return 0;

}

bool Sphere::isInsideAt(const Vector3 &locallocation) const {
    return (locallocation.lengthSq()<radiussq)^inversed;
}

void Sphere::calculateBoundingBox() {
    boundingbox = new BoundingBox(Vector3(-radius,-radius,-radius),Vector3(radius,radius,radius));
    boundingbox->transformate(*this);
}

void Sphere::prepareToRender() {

   if (transformation == 0) {
        delete(boundingbox);
        boundingbox = 0;
    }

}

void Sphere::SphereIntersection::next() {
    if (currenttime != INTERSECTION_INFINIT_TIME) {
        currenttime = secondtime;
        secondtime = INTERSECTION_INFINIT_TIME;
    }
}

Vector3 Sphere::SphereIntersection::normal() {
    return ray.positionAtTime(currenttime);
}

